@@ -24,6 +24,8 @@
 
 #include <glib/gi18n.h>
 
+#include "_stdint.h"
+
 #include <dvdread/dvd_reader.h>
 #include <dvdread/ifo_types.h>
 #include <dvdread/ifo_read.h>
@@ -835,6 +837,7 @@ dd_audio_format_supported (guint audio_format)
 	static gboolean  checked; /* FALSE */
 	static gboolean  lpcm_ok; /* FALSE */
 	static gboolean  dts_ok;  /* FALSE */
+	static gboolean  mp3_ok;  /* FALSE */
 
 	if (audio_format == AUDIO_FORMAT_AC3)
 		return TRUE;
@@ -859,6 +862,13 @@ dd_audio_format_supported (guint audio_format)
 				th_log ("dtsdec plugin available");
 				dts_ok = TRUE;
 			} else th_log ("dtsdec plugin not available");
+
+			/* ffdec_mp3 isn't tested well enough and mad is hardcoded in th-job.c */
+			if (th_utils_check_plugin_version ("mad", "mad", 0, 8, 7, 1))
+			{
+				th_log ("mad plugin available");
+				mp3_ok = TRUE;
+			} else th_log ("mad plugin not available");
 		}
 		checked = TRUE;
 	}
@@ -869,6 +879,9 @@ dd_audio_format_supported (guint audio_format)
 	if (audio_format == AUDIO_FORMAT_DTS)
 		return dts_ok;
 
+	if (audio_format == AUDIO_FORMAT_MPEG1)
+		return mp3_ok;
+
 	return FALSE;
 }
 
